class ConfigurationException(Exception):
    """
    An exception for Configuration errors
    """
    message = 'There was an error in the configuration'


class NewRelicAPIServerException(Exception):
    """
    An exception for New Relic server errors
    """
    message = 'There was an error from New Relic'



import json
from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.alert_policies import AlertPolicies


class NRAlertPoliciesTests(TestCase):
    def setUp(self):
        super(NRAlertPoliciesTests, self).setUp()
        self.policies = AlertPolicies(api_key='dummy_key')

        self.policies_list_response = {
            "alert_policies": [
                {
                    "id": 12345,
                    "type": "server",
                    "name": "Default Server Policy",
                    "enabled": True,
                    "conditions": [
                        {
                            "id": 347535,
                            "type": "disk_io",
                            "severity": "caution",
                            "threshold": 70,
                            "trigger_minutes": 20,
                            "enabled": True
                        },
                        {
                            "id": 347536,
                            "type": "disk_io",
                            "severity": "critical",
                            "threshold": 90,
                            "trigger_minutes": 15,
                            "enabled": True
                        },
                        {
                            "id": 347537,
                            "type": "fullest_disk",
                            "severity": "caution",
                            "threshold": 70,
                            "trigger_minutes": 10,
                            "enabled": True
                        },
                        {
                            "id": 347538,
                            "type": "fullest_disk",
                            "severity": "critical",
                            "threshold": 90,
                            "trigger_minutes": 5,
                            "enabled": True
                        },
                        {
                            "id": 347539,
                            "type": "memory",
                            "severity": "caution",
                            "threshold": 80,
                            "trigger_minutes": 10,
                            "enabled": True
                        },
                        {
                            "id": 347540,
                            "type": "memory",
                            "severity": "critical",
                            "threshold": 95,
                            "trigger_minutes": 5,
                            "enabled": True
                        },
                        {
                            "id": 347541,
                            "type": "cpu",
                            "severity": "caution",
                            "threshold": 60,
                            "trigger_minutes": 20,
                            "enabled": True
                        },
                        {
                            "id": 347542,
                            "type": "cpu",
                            "severity": "critical",
                            "threshold": 90,
                            "trigger_minutes": 15,
                            "enabled": True
                        },
                        {
                            "id": 347543,
                            "type": "server_downtime",
                            "severity": "downtime",
                            "trigger_minutes": 5,
                            "enabled": True
                        }
                    ],
                    "links": {
                        "notification_channels": [
                            333444
                        ],
                        "servers": [
                            1234567,
                            2345678,
                            3456789,
                            4567890,
                            5678901,
                            6789012
                        ]
                    }
                }
            ]
        }
        self.policy_show_response = {
            'alert_policy': self.policies_list_response['alert_policies'][0]
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test alert policies .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.policies_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.policies.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_ids(self, mock_get):
        """
        Test alert policies .list() with filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.policies_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.policies.list(filter_ids=[12345])

        self.assertIsInstance(response, dict)
        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/alert_policies.json',
            headers=self.policies.headers,
            params='filter[ids]=12345'
        )

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test alert policies .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.policies.list()

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test alert policies .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.policy_show_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.policies.show(id=333112)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test alert policies .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.policies.show(id=333114)

    @patch.object(requests, 'put')
    def test_update(self, mock_put):
        """
        Test alert policies .update() calls put with correct parameters
        """
        self.policies.update(id=333114, policy_update=self.policy_show_response)

        mock_put.assert_called_once_with(
            url='https://api.newrelic.com/v2/alert_policies/333114.json',
            headers=self.policies.headers,
            data=json.dumps(self.policy_show_response)
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.plugins import Plugins


class NRPluginsTests(TestCase):
    def setUp(self):
        super(NRPluginsTests, self).setUp()
        self.plugins = Plugins(api_key='dummy_key')

        self.plugins_list_response = {
            'plugins': [
                {
                    "id": 2227,
                    "name": "SendGrid",
                    "guid": "com.SendGrid",
                    "publisher": "SendGrid",
                    "details": {
                        "description": "Email Delivery. Simplified.\r\n\r\n"
                                       "SendGrid is a simple email infrastructure in the cloud.Our platform allows "
                                       "developers to easily integrate email functionality into their web and mobile "
                                       "applications. Through APIs or SMTP relay, you can send and receive emails "
                                       "while retrieving detailed email statistics. Our users enjoy increased "
                                       "deliverability of email, on demand scalability and more time to focus on their "
                                       "product instead of taking care of email servers.  \r\n\r\n"
                                       "New Relic's SendGrid plugin provides statistical monitoring and a simple 'at a "
                                       "glance' health check of ones email traffic. The plugin includes real time "
                                       "information on email performance including deliverability, engagement, and "
                                       "compliance factors.  \r\n\r\n"
                                       "### Requirements\r\n\r\n"
                                       "* a SendGrid account\r\n"
                                       "* a New Relic account",
                        "is_public": None,
                        "created_at": "2013-09-19T10:55:06-07:00",
                        "updated_at": "2014-01-27T14:15:35-08:00",
                        "last_published_at": None,
                        "has_unpublished_changes": True,
                        "branding_image_url": "http://static.sendgrid.com.s3.amazonaws.com/images/64X64.png",
                        "upgraded_at": "2013-06-19T10:55:06-07:00",
                        "short_name": "SendGrid",
                        "publisher_about_url": "http://sendgrid.com/",
                        "publisher_support_url": "http://support.sendgrid.com/home",
                        "download_url": "http://sendgrid.com/app/appSettings/type/newrelic/id/22",
                        "first_edited_at": None,
                        "last_edited_at": None,
                        "first_published_at": None,
                        "published_version": None
                    },
                    "summary_metrics": []
                }
            ]
        }
        self.plugins_show_response = {
            'plugin': self.plugins_list_response['plugins'][0]
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test plugins .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.plugins_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.plugins.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_filter_ids(self, mock_get):
        """
        Test plugins .list() with filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.plugins_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.plugins.list(filter_ids=[2227])

        self.assertIsInstance(response, dict)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/plugins.json',
            headers=self.plugins.headers,
            params='filter[ids]=2227'
        )

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test plugins .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.plugins.list()

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test plugins .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.plugins_show_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.plugins.show(id=2227)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test plugins .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.plugins.show(id=2227)



from collections import OrderedDict
from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.servers import Servers


class NRServersTests(TestCase):
    def setUp(self):
        super(NRServersTests, self).setUp()
        self.server = Servers(api_key='dummy_key')

        server_response = {
            "id": 1234567,
            "account_id": 12345,
            "name": "ip-10-0-13-182",
            "host": "ip-10-0-13-182",
            "reporting": True,
            "last_reported_at": "2014-06-24T18:52:09+00:00",
            "summary": {
                "cpu": 14.1,
                "cpu_stolen": 0,
                "disk_io": 0.07,
                "memory": 69,
                "memory_used": 2712666112,
                "memory_total": 3932160000,
                "fullest_disk": 30.7,
                "fullest_disk_free": 5724000000
            }
        }
        self.list_success_response = {
            'servers': [
                server_response,
            ],
        }

        self.show_success_response = {
            'server': server_response,
        }

        self.metric_name_response = {
            "metrics": [
                {
                    "name": "Agent/MetricsReported/count",
                    "values": [
                        "average_response_time",
                        "calls_per_minute",
                        "call_count",
                        "min_response_time",
                        "max_response_time",
                        "average_exclusive_time",
                        "average_value",
                        "requests_per_minute",
                        "standard_deviation"
                    ]
                },
                {
                    "name": "ProcessSamples/messagebus/dbus-daemon",
                    "values": [
                        "average_response_time",
                        "calls_per_minute",
                        "call_count",
                        "min_response_time",
                        "max_response_time",
                        "average_exclusive_time",
                        "average_value",
                        "requests_per_minute",
                        "standard_deviation"
                    ]
                }
            ]
        }

        self.metric_data_response = {
            "metric_data": {
                "from": "2014-06-24T19:41:47+00:00",
                "to": "2014-06-24T20:11:47+00:00",
                "metrics": [
                    {
                        "name": "Agent/MetricsReported/count",
                        "timeslices": [
                            {
                                "from": "2014-06-24T19:40:00+00:00",
                                "to": "2014-06-24T20:09:59+00:00",
                                "values": {
                                    "average_response_time": 76,
                                    "calls_per_minute": 1,
                                    "call_count": 30,
                                    "min_response_time": 76,
                                    "max_response_time": 76,
                                    "average_exclusive_time": 76,
                                    "average_value": 0.076,
                                    "requests_per_minute": 1,
                                    "standard_deviation": 0.0107
                                }
                            }
                        ]
                    }
                ]
            }
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test servers .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.server.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_filter_ids(self, mock_get):
        """
        Test servers .list() with filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.server.list(filter_ids=[1234567])

        self.assertIsInstance(response, dict)
        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/servers.json',
            headers=self.server.headers,
            params='filter[ids]=1234567'
        )

    @patch.object(requests, 'get')
    def test_list_success_with_filter_labels(self, mock_get):
        """
        Test servers .list() with filter_labels
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        # Use ordered dict to guarantee ordering of labels in query param
        labels = OrderedDict((('Type1', 'Value1'), ('Type2', 'Value2')))
        response = self.server.list(filter_labels=labels)

        self.assertIsInstance(response, dict)
        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/servers.json',
            headers=self.server.headers,
            params='filter[labels]=Type1:Value1;Type2:Value2'
        )

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test servers .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.server.list()

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test servers .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.show_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.server.show(id=1234567)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test servers .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.server.show(id=1234567)

    @patch.object(requests, 'get')
    @patch.object(requests, 'put')
    def test_update_success(self, mock_put, mock_get):
        """
        Test servers .update() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.show_success_response
        mock_get.return_value = mock_response
        mock_put.return_value = mock_response

        # Call the method
        response = self.server.update(id=1234568, name='New Name')

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    @patch.object(requests, 'put')
    def test_update_failure(self, mock_put, mock_get):
        """
        Test servers .update() failure
        """
        mock_response1 = Mock(name='response')
        mock_response1.json.return_value = self.show_success_response
        mock_get.return_value = mock_response1

        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_put.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.server.update(id=1234567)

    @patch.object(requests, 'delete')
    def test_delete_success(self, mock_delete):
        """
        Test servers .delete() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.show_success_response
        mock_delete.return_value = mock_response

        # Call the method
        response = self.server.delete(id=1234567)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'delete')
    def test_delete_failure(self, mock_delete):
        """
        Test servers .delete() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_delete.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.server.delete(id=1234567)

    @patch.object(requests, 'get')
    def test_metric_names(self, mock_get):
        """
        Test servers .metric_names()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_name_response
        mock_get.return_value = mock_response

        response = self.server.metric_names(id=1234567)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_without_values(self, mock_get):
        """
        Test servers .metric_data() without values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.server.metric_data(id=1234567, names=['Agent/MetricsReported/count'], summarize=True)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_with_values(self, mock_get):
        """
        Test servers .metric_data() with values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.server.metric_data(
            id=1234567,
            names=['Agent/MetricsReported/count'],
            values=['call_count'],
            summarize=True
        )

        self.assertIsInstance(response, dict)



from .base import Resource


class Labels(Resource):
    """
    An interface for interacting with the NewRelic label API.
    """
    def list(self, page=None):
        """
        This API endpoint returns a paginated list of the Labels
        associated with your New Relic account.

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "labels": [
                    {
                        "key": "string",
                        "category": "string",
                        "name": "string",
                        "links": {
                            "applications": [
                                "integer"
                            ],
                            "servers": [
                                "integer"
                            ]
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/labels.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/labels.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """

        filters = [
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}labels.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def create(self, name, category, applications=None, servers=None):
        """
        This API endpoint will create a new label with the provided name and
            category

        :type name: str
        :param name: The name of the label

        :type category: str
        :param category: The Category

        :type applications: list of int
        :param applications: An optional list of application ID's

        :type servers: list of int
        :param servers: An optional list of server ID's

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "label": {
                    "key": "string",
                    "category": "string",
                    "name": "string",
                    "links": {
                        "applications": [
                            "integer"
                        ],
                        "servers": [
                            "integer"
                        ]
                    }
                }
            }

        """

        data = {
            "label": {
                "category": category,
                "name": name,
                "links": {
                    "applications": applications or [],
                    "servers": servers or []
                }
            }
        }

        return self._put(
            url='{0}labels.json'.format(self.URL),
            headers=self.headers,
            data=data
        )

    def delete(self, key):
        """
        When applications are provided, this endpoint will remove those
            applications from the label.

        When no applications are provided, this endpoint will remove the label.

        :type key: str
        :param key: Label key. Example: 'Language:Java'

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "label": {
                    "key": "string",
                    "category": "string",
                    "name": "string",
                    "links": {
                        "applications": [
                            "integer"
                        ],
                        "servers": [
                            "integer"
                        ]
                    }
                }
            }

        """
        return self._delete(
            url='{url}labels/labels/{key}.json'.format(
                url=self.URL,
                key=key),
            headers=self.headers,
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.application_instances import ApplicationInstances


class NRApplicationInstancesTests(TestCase):
    def setUp(self):
        super(NRApplicationInstancesTests, self).setUp()
        self.app_instances = ApplicationInstances(api_key='dummy_key')

        app_response = {
            "id": 1234567,
            "application_name": "Developer Application",
            "host": "ip-10-1-19-12",
            "port": 0,
            "language": "python",
            "health_status": "green",
            "application_summary": {
                "response_time": 195,
                "throughput": 22.2,
                "error_rate": 0,
                "apdex_score": 0.98
            },
            "links": {
                "application": 2345678,
                "application_host": 3456789,
                "server": 4567890
            }
        }
        links = {
            "application_instance.application": "/v2/applications/{application_id}",
            "application_instance.application_host": "/v2/application/{application_id}/hosts/{instance_id}",
            "application_instance.server": "/v2/servers/{server_id}"
        }
        self.list_success_response = {
            'application_instances': [
                app_response,
            ],
            'links': links
        }

        self.show_success_response = {
            'application_instance': app_response,
            'links': links
        }

        self.metric_name_response = {
            "metrics": [
                {
                    "values": [
                        "average_response_time",
                        "calls_per_minute",
                        "call_count",
                        "min_response_time",
                        "max_response_time",
                        "average_exclusive_time",
                        "average_value",
                        "requests_per_minute",
                        "standard_deviation"
                    ],
                    "name": "Agent/MetricsReported/count"
                },
                {
                    "values": [
                        "s",
                        "t",
                        "f",
                        "count",
                        "score",
                        "value",
                        "threshold",
                        "threshold_min"
                    ],
                    "name": "Apdex"
                }
            ]
        }
        self.metric_data_response = {
            "metric_data": {
                "metrics": [
                    {
                        "timeslices": [
                            {
                                "values": {
                                    "total_time": 15700,
                                    "percent": 0.871,
                                    "average_time": 180
                                },
                                "to": "2014-06-24T19:42:59+00:00",
                                "from": "2014-06-24T19:13:00+00:00"
                            }
                        ],
                        "name": "CPU/User Time"
                    }
                ],
                "to": "2014-06-24T19:44:29+00:00",
                "from": "2014-06-24T19:14:29+00:00"
            }
        }

        self.application_id = 1234567

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test application instances .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.app_instances.list(application_id=2345678)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_filter_ids(self, mock_get):
        """
        Test application instances .list() with_filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.app_instances.list(application_id=2345678, filter_ids=[1234567])

        self.assertIsInstance(response, dict)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/applications/2345678/instances.json',
            headers=self.app_instances.headers,
            params='filter[ids]=1234567'
        )

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test application instances .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.app_instances.list(application_id=1234567)

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test application instances .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.show_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.app_instances.show(application_id=1234567, instance_id=2345678)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test application instances .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.app_instances.show(application_id=1234567, instance_id=2345678)

    @patch.object(requests, 'get')
    def test_metric_names(self, mock_get):
        """
        Test application instances .metric_names()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_name_response
        mock_get.return_value = mock_response

        response = self.app_instances.metric_names(
            application_id=1234567,
            instance_id=2345678
        )

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_without_values(self, mock_get):
        """
        Test application instances .metric_data() without a values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.app_instances.metric_data(
            application_id=1234567,
            instance_id=2345678,
            names=['CPU/User Time'],
            summarize=True
        )

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_with_values(self, mock_get):
        """
        Test application instances .metric_data() with a values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.app_instances.metric_data(
            application_id=1234567,
            instance_id=2345678,
            names=['CPU/User Time'],
            values=['percent'],
            summarize=True
        )

        self.assertIsInstance(response, dict)



from .base import Resource


class Servers(Resource):
    """
    An interface for interacting with the NewRelic server API.
    """
    def list(self, filter_name=None, filter_ids=None, filter_labels=None, page=None):
        """
        This API endpoint returns a paginated list of the Servers
        associated with your New Relic account. Servers can be filtered
        by their name or by a list of server IDs.

        :type filter_name: str
        :param filter_name: Filter by server name

        :type filter_ids: list of ints
        :param filter_ids: Filter by server ids

        :type filter_labels: dict of label type: value pairs
        :param filter_labels: Filter by server labels

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "servers": [
                    {
                        "id": "integer",
                        "account_id": "integer",
                        "name": "string",
                        "host": "string",
                        "reporting": "boolean",
                        "last_reported_at": "time",
                        "summary": {
                            "cpu": "float",
                            "cpu_stolen": "float",
                            "disk_io": "float",
                            "memory": "float",
                            "memory_used": "integer",
                            "memory_total": "integer",
                            "fullest_disk": "float",
                            "fullest_disk_free": "integer"
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/servers.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/servers.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        label_param = ''

        if filter_labels:
            label_param = ';'.join(['{}:{}'.format(label, value) for label, value in filter_labels.items()])

        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'filter[labels]={0}'.format(label_param) if filter_labels else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}servers.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single Server, identified its ID.

        :type id: int
        :param id: Server ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "server": {
                    "id": "integer",
                    "account_id": "integer",
                    "name": "string",
                    "host": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "summary": {
                        "cpu": "float",
                        "cpu_stolen": "float",
                        "disk_io": "float",
                        "memory": "float",
                        "memory_used": "integer",
                        "memory_total": "integer",
                        "fullest_disk": "float",
                        "fullest_disk_free": "integer"
                    }
                }
            }

        """
        return self._get(
            url='{0}servers/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )

    def update(self, id, name=None):
        """
        Updates any of the optional parameters of the server

        :type id: int
        :param id: Server ID

        :type name: str
        :param name: The name of the server

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "server": {
                    "id": "integer",
                    "account_id": "integer",
                    "name": "string",
                    "host": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "summary": {
                        "cpu": "float",
                        "cpu_stolen": "float",
                        "disk_io": "float",
                        "memory": "float",
                        "memory_used": "integer",
                        "memory_total": "integer",
                        "fullest_disk": "float",
                        "fullest_disk_free": "integer"
                    }
                }
            }

        """
        nr_data = self.show(id)['server']

        data = {
            'server': {
                'name': name or nr_data['name'],
            }
        }

        return self._put(
            url='{0}servers/{1}.json'.format(self.URL, id),
            headers=self.headers,
            data=data
        )

    def delete(self, id):
        """
        This API endpoint deletes an server and all of its reported data.

        WARNING: Only servers that have stopped reporting can be deleted.
            This is an irreversible process which will delete all reported
            data for this server.

        :type id: int
        :param id: Server ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "server": {
                    "id": "integer",
                    "account_id": "integer",
                    "name": "string",
                    "host": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "summary": {
                        "cpu": "float",
                        "cpu_stolen": "float",
                        "disk_io": "float",
                        "memory": "float",
                        "memory_used": "integer",
                        "memory_total": "integer",
                        "fullest_disk": "float",
                        "fullest_disk_free": "integer"
                    }
                }
            }

        """
        return self._delete(
            url='{0}servers/{1}.json'.format(
                self.URL,
                id),
            headers=self.headers,
        )

    def metric_names(self, id, name=None, page=None):
        """
        Return a list of known metrics and their value names for the given resource.

        :type id: int
        :param id: Server ID

        :type name: str
        :param name: Filter metrics by name

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "metrics": [
                    {
                        "name": "string",
                        "values": [
                            "string"
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/servers/{server_id}/metrics.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/servers/{server_id}/metrics.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        params = [
            'name={0}'.format(name) if name else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}servers/{1}/metrics.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )

    def metric_data(
            self, id, names, values=None, from_dt=None, to_dt=None,
            summarize=False):
        """
        This API endpoint returns a list of values for each of the requested
        metrics. The list of available metrics can be returned using the Metric
        Name API endpoint. Metric data can be filtered by a number of
        parameters, including multiple names and values, and by time range.
        Metric names and values will be matched intelligently in the
        background. You can also retrieve a summarized data point across the
        entire time range selected by using the summarize parameter.

        **Note** All times sent and received are formatted in UTC. The default
        time range is the last 30 minutes.

        :type id: int
        :param id: Server ID

        :type names: list of str
        :param names: Retrieve specific metrics by name

        :type values: list of str
        :param values: Retrieve specific metric values

        :type from_dt: datetime
        :param from_dt: Retrieve metrics after this time

        :type to_dt: datetime
        :param to_dt: Retrieve metrics before this time

        :type summarize: bool
        :param summarize: Summarize the data

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "metric_data": {
                    "from": "time",
                    "to": "time",
                    "metrics": [
                        {
                            "name": "string",
                            "timeslices": [
                                {
                                    "from": "time",
                                    "to": "time",
                                    "values": "hash"
                                }
                            ]
                        }
                    ]
                }
            }

        """
        params = [
            'from={0}'.format(from_dt) if from_dt else None,
            'to={0}'.format(to_dt) if to_dt else None,
            'summarize=true' if summarize else None
        ]

        params += ['names[]={0}'.format(name) for name in names]
        if values:
            params += ['values[]={0}'.format(value) for value in values]

        return self._get(
            url='{0}servers/{1}/metrics/data.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )



from .base import Resource


class ApplicationHosts(Resource):
    """
    An interface for interacting with the New Relic Application Hosts API.
    """
    def list(
            self, application_id, filter_hostname=None, filter_ids=None,
            page=None):
        """
        This API endpoint returns a paginated list of hosts associated with the
        given application.

        Application hosts can be filtered by hostname, or the list of
        application host IDs.

        :type application_id: int
        :param application_id: Application ID

        :type filter_hostname: str
        :param filter_hostname: Filter by server hostname

        :type filter_ids: list of ints
        :param filter_ids: Filter by application host ids

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "application_hosts": [
                    {
                        "id": "integer",
                        "application_name": "string",
                        "host": "string",
                        "language": "integer",
                        "health_status": "string",
                        "application_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "error_rate": "float",
                            "apdex_score": "float"
                        },
                        "end_user_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "apdex_score": "float"
                        },
                        "links": {
                            "application": "integer",
                            "application_instances": [
                                "integer"
                            ],
                            "server": "integer"
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/applications/{application_id}/hosts.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/applications/{application_id}/hosts.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[hostname]={0}'.format(filter_hostname) if filter_hostname else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{root}applications/{application_id}/hosts.json'.format(
                root=self.URL,
                application_id=application_id
            ),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, application_id, host_id):
        """
        This API endpoint returns a single application host, identified by its
        ID.

        :type application_id: int
        :param application_id: Application ID

        :type host_id: int
        :param host_id: Application host ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "application_host": {
                    "id": "integer",
                    "application_name": "string",
                    "host": "string",
                    "language": "integer",
                    "health_status": "string",
                    "application_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "error_rate": "float",
                        "apdex_score": "float"
                    },
                    "end_user_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "apdex_score": "float"
                    },
                    "links": {
                        "application": "integer",
                        "application_instances": [
                            "integer"
                        ],
                        "server": "integer"
                    }
                }
            }

        """
        return self._get(
            url='{root}applications/{application_id}/hosts/{host_id}.json'.format(
                root=self.URL,
                application_id=application_id,
                host_id=host_id
            ),
            headers=self.headers,
        )

    def metric_names(self, application_id, host_id, name=None, page=None):
        """
        Return a list of known metrics and their value names for the given resource.

        :type application_id: int
        :param application_id: Application ID

        :type host_id: int
        :param host_id: Application Host ID

        :type name: str
        :param name: Filter metrics by name

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "metrics": [
                    {
                        "name": "string",
                        "values": [
                            "string"
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/\
applications/{application_id}/hosts/{host_id}/metrics.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/\
applications/{application_id}/hosts/{host_id}/metrics.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        params = [
            'name={0}'.format(name) if name else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{root}applications/{application_id}/hosts/{host_id}/metrics.json'.format(
                root=self.URL,
                application_id=application_id,
                host_id=host_id
            ),
            headers=self.headers,
            params=self.build_param_string(params)
        )

    def metric_data(
            self, application_id, host_id, names, values=None, from_dt=None, to_dt=None,
            summarize=False):
        """
        This API endpoint returns a list of values for each of the requested
        metrics. The list of available metrics can be returned using the Metric
        Name API endpoint. Metric data can be filtered by a number of
        parameters, including multiple names and values, and by time range.
        Metric names and values will be matched intelligently in the
        background. You can also retrieve a summarized data point across the
        entire time range selected by using the summarize parameter.

        **Note** All times sent and received are formatted in UTC. The default
        time range is the last 30 minutes.

        :type application_id: int
        :param application_id: Application ID

        :type host_id: int
        :param host_id: Application Host ID

        :type names: list of str
        :param names: Retrieve specific metrics by name

        :type values: list of str
        :param values: Retrieve specific metric values

        :type from_dt: datetime
        :param from_dt: Retrieve metrics after this time

        :type to_dt: datetime
        :param to_dt: Retrieve metrics before this time

        :type summarize: bool
        :param summarize: Summarize the data

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "metric_data": {
                    "from": "time",
                    "to": "time",
                    "metrics": [
                        {
                            "name": "string",
                            "timeslices": [
                                {
                                    "from": "time",
                                    "to": "time",
                                    "values": "hash"
                                }
                            ]
                        }
                    ]
                }
            }

        """
        params = [
            'from={0}'.format(from_dt) if from_dt else None,
            'to={0}'.format(to_dt) if to_dt else None,
            'summarize=true' if summarize else None
        ]

        params += ['names[]={0}'.format(name) for name in names]
        if values:
            params += ['values[]={0}'.format(value) for value in values]

        return self._get(
            url='{url}applications/{application_id}/hosts/{host_id}/metrics/data.json'.format(
                url=self.URL,
                application_id=application_id,
                host_id=host_id,
            ),
            headers=self.headers,
            params=self.build_param_string(params)
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.labels import Labels


class NRLabelsTests(TestCase):
    def setUp(self):
        super(NRLabelsTests, self).setUp()
        self.label = Labels(api_key='dummy_key')

        label = {
            "key": "Ambition:Production",
            "category": "Ambition",
            "name": "Production",
            "links": {
                "applications": [],
                "servers": [
                    1234567
                ]
            }
        }
        self.list_success_response = {
            'labels': [
                label,
            ],
        }
        self.create_success_response = {
            'label': label,
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test labels .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.label.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test labels .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.label.list()

    @patch.object(requests, 'put')
    def test_create_success(self, mock_put):
        """
        Test labels .create() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.create_success_response
        mock_put.return_value = mock_response

        # Call the method
        response = self.label.create(name='Production', category='Ambition')

        self.assertIsInstance(response, dict)
        self.assertEqual(response['label']['key'], 'Ambition:Production')

    @patch.object(requests, 'put')
    def test_create_failure(self, mock_put):
        """
        Test labels .create() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_put.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.label.create(name='Production', category='Ambition')

    @patch.object(requests, 'delete')
    def test_delete_success(self, mock_delete):
        """
        Test labels .delete() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.create_success_response
        mock_delete.return_value = mock_response

        # Call the method
        response = self.label.delete(key='Ambition:Production')

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'delete')
    def test_delete_failure(self, mock_delete):
        """
        Test labels .delete() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_delete.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.label.delete(key='Ambition:Production')



from .base import Resource


class Users(Resource):
    """
    An interface for interacting with the NewRelic user API.
    """
    def list(self, filter_email=None, filter_ids=None, page=None):
        """
        This API endpoint returns a paginated list of the Users
        associated with your New Relic account. Users can be filtered
        by their email or by a list of user IDs.

        :type filter_email: str
        :param filter_email: Filter by user email

        :type filter_ids: list of ints
        :param filter_ids: Filter by user ids

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "users": [
                    {
                        "id": "integer",
                        "first_name": "string",
                        "last_name": "string",
                        "email": "string",
                        "role": "string"
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/users.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/users.json?page=2",
                        "rel": "next"
                    }
                }
            }
        """
        filters = [
            'filter[email]={0}'.format(filter_email) if filter_email else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}users.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single User, identified its ID.

        :type id: int
        :param id: User ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "user": {
                    "id": "integer",
                    "first_name": "string",
                    "last_name": "string",
                    "email": "string",
                    "role": "string"
                }
            }

        """
        return self._get(
            url='{0}users/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.users import Users


class NRUsersTests(TestCase):
    def setUp(self):
        super(NRUsersTests, self).setUp()
        self.user = Users(api_key='dummy_key')

        self.user_list_response = {
            'users': [
                {
                    "id": 333113,
                    "first_name": "Josh",
                    "last_name": "Marlow",
                    "email": "josh.marlow@ambition.com",
                    "role": "user"
                },
                {
                    "id": 333112,
                    "first_name": "Micah",
                    "last_name": "Hausler",
                    "email": "micah.hausler@ambition.com",
                    "role": "admin"
                },
                {
                    "id": 333111,
                    "first_name": "Travis",
                    "last_name": "Truett",
                    "email": "travis.truett@ambition.com",
                    "role": "owner"
                }
            ]
        }
        self.user_show_response = self.user_list_response['users'][1]

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test users .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.user_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.user.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_ids(self, mock_get):
        """
        Test users .list() with filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.user_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.user.list(filter_ids=[333113, 333112, 333111])

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test users .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.user.list()

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test users .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.user_show_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.user.show(id=333112)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test users .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.user.show(id=333114)



from .base import Resource


class Plugins(Resource):
    """
    An interface for interacting with the NewRelic Plugins API.
    """
    def list(self, filter_guid=None, filter_ids=None, detailed=None, page=None):
        """
        This API endpoint returns a paginated list of the plugins associated
        with your New Relic account.

        Plugins can be filtered by their name or by a list of IDs.

        :type filter_guid: str
        :param filter_guid: Filter by name

        :type filter_ids: list of ints
        :param filter_ids: Filter by user ids

        :type detailed: bool
        :param detailed: Include all data about a plugin

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "plugins": [
                    {
                        "id": "integer",
                        "name": "string",
                        "guid": "string",
                        "publisher": "string",
                        "details": {
                            "description": "integer",
                            "is_public": "string",
                            "created_at": "time",
                            "updated_at": "time",
                            "last_published_at": "time",
                            "has_unpublished_changes": "boolean",
                            "branding_image_url": "string",
                            "upgraded_at": "time",
                            "short_name": "string",
                            "publisher_about_url": "string",
                            "publisher_support_url": "string",
                            "download_url": "string",
                            "first_edited_at": "time",
                            "last_edited_at": "time",
                            "first_published_at": "time",
                            "published_version": "string"
                        },
                        "summary_metrics": [
                            {
                                "id": "integer",
                                "name": "string",
                                "metric": "string",
                                "value_function": "string",
                                "thresholds": {
                                    "caution": "float",
                                    "critical": "float"
                                },
                                "values": {
                                    "raw": "float",
                                    "formatted": "string"
                                }
                            }
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/plugins.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/plugins.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[guid]={0}'.format(filter_guid) if filter_guid else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'detailed={0}'.format(detailed) if detailed is not None else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}plugins.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id, detailed=None):
        """
        This API endpoint returns a single Key transaction, identified its ID.

        :type id: int
        :param id: Key transaction ID

        :type detailed: bool
        :param detailed:

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "plugin": {
                    "id": "integer",
                    "name": "string",
                    "guid": "string",
                    "publisher": "string",
                    "details": {
                        "description": "integer",
                        "is_public": "string",
                        "created_at": "time",
                        "updated_at": "time",
                        "last_published_at": "time",
                        "has_unpublished_changes": "boolean",
                        "branding_image_url": "string",
                        "upgraded_at": "time",
                        "short_name": "string",
                        "publisher_about_url": "string",
                        "publisher_support_url": "string",
                        "download_url": "string",
                        "first_edited_at": "time",
                        "last_edited_at": "time",
                        "first_published_at": "time",
                        "published_version": "string"
                    },
                    "summary_metrics": [
                        {
                            "id": "integer",
                            "name": "string",
                            "metric": "string",
                            "value_function": "string",
                            "thresholds": {
                                "caution": "float",
                                "critical": "float"
                            },
                            "values": {
                                "raw": "float",
                                "formatted": "string"
                            }
                        }
                    ]
                }
            }

        """
        filters = [
            'detailed={0}'.format(detailed) if detailed is not None else None,
        ]
        return self._get(
            url='{root}plugins/{id}.json'.format(
                root=self.URL,
                id=id
            ),
            headers=self.headers,
            params=self.build_param_string(filters) or None
        )



from .base import Resource


class KeyTransactions(Resource):
    """
    An interface for interacting with the NewRelic key transactions API.
    """
    def list(self, filter_name=None, filter_ids=None, page=None):
        """
        This API endpoint returns a paginated list of the key transactions
        associated with your New Relic account.

        Key transactions can be filtered by their name or by a list of IDs.

        :type filter_name: str
        :param filter_name: Filter by name

        :type filter_ids: list of ints
        :param filter_ids: Filter by user ids

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "key_transactions": [
                    {
                        "id": "integer",
                        "name": "string",
                        "transaction_name": "string",
                        "application_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "error_rate": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "end_user_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "links": {
                            "application": "integer"
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/key_transactions.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/key_transactions.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{0}key_transactions.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single Key transaction, identified its ID.

        :type id: int
        :param id: Key transaction ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "key_transaction": {
                    "id": "integer",
                    "name": "string",
                    "transaction_name": "string",
                    "application_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "error_rate": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "end_user_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "links": {
                        "application": "integer"
                    }
                }
            }

        """
        return self._get(
            url='{root}key_transactions/{id}.json'.format(
                root=self.URL,
                id=id
            ),
            headers=self.headers,
        )



from unittest import TestCase

from mock import patch
import requests

from newrelic_api.notification_channels import NotificationChannels


class NRNotificationChannelsTests(TestCase):
    def setUp(self):
        super(NRNotificationChannelsTests, self).setUp()
        self.channels = NotificationChannels(api_key='dummy_key')

        self.list_response = {
            "notification_channels": [
                {
                    "id": 111222,
                    "type": "user",
                    "downtime_only": False,
                    "mobile_alerts": False,
                    "email_alerts": True,
                    "links": {
                        "user": 222333
                    }
                }
            ]
        }

        self.show_response = {
            'notification_channel': self.list_response['notification_channels'][0]
        }

    @patch.object(requests, 'get')
    def test_list(self, mock_get):
        """
        Test notification channels .list()
        """
        self.channels.list(filter_type=['user'], page=0)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/notification_channels.json',
            headers=self.channels.headers,
            params='filter[type]=user'
        )

    @patch.object(requests, 'get')
    def test_list_with_filter_ids(self, mock_get):
        """
        Test notification channels .list() with filter_ids
        """
        self.channels.list(filter_type=['user'], filter_ids=[111222], page=0)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/notification_channels.json',
            headers=self.channels.headers,
            params='filter[type]=user&filter[ids]=111222'
        )

    @patch.object(requests, 'get')
    def test_show(self, mock_get):
        """
        Test notification channels .show()
        """
        self.channels.show(id=11122)
        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/notification_channels/11122.json',
            headers=self.channels.headers,
        )



from .base import Resource


class AlertPolicies(Resource):
    """
    An interface for interacting with the NewRelic Alert Policies API.
    """
    def list(
            self, filter_name=None, filter_type=None, filter_ids=None,
            filter_enabled=None, page=None):
        """
        This API endpoint returns a paginated list of the alert policies
        associated with your New Relic account. Alert policies can be filtered
        by their name, list of IDs, type (application, key_transaction, or
        server) or whether or not policies are archived (defaults to filtering
        archived policies).

        :type filter_name: str
        :param filter_name: Filter by name

        :type filter_type: list of str
        :param filter_type: Filter by policy types.

        :type filter_ids: list of int
        :param filter_ids: Filter by policy IDs

        :type filter_enabled: bool
        :param filter_enabled: Select only enabled/disabled policies (default: both)

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "alert_policies": [
                    {
                        "id": "integer",
                        "type": "string",
                        "name": "string",
                        "enabled": "boolean",
                        "conditions": [
                            {
                                "id": "integer",
                                "type": "string",
                                "severity": "string",
                                "threshold": "float",
                                "trigger_minutes": "integer",
                                "enabled": "boolean"
                            }
                        ],
                        "links": {
                            "notification_channels": [
                                "integer"
                            ],
                            "applications": [
                                "integer"
                            ],
                            "key_transactions": [
                                "integer"
                            ],
                            "servers": [
                                "integer"
                            ]
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/alert_policies.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/alert_policies.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[type]={0}'.format(','.join(filter_type)) if filter_type else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'filter[enabled]={0}'.format(filter_enabled) if filter_enabled in [True, False] else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}alert_policies.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single alert policy, identified by ID.

        :type id: int
        :param id: Alert policy ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "alert_policy": {
                    "id": "integer",
                    "type": "string",
                    "name": "string",
                    "enabled": "boolean",
                    "conditions": [
                        {
                            "id": "integer",
                            "type": "string",
                            "severity": "string",
                            "threshold": "float",
                            "trigger_minutes": "integer",
                            "enabled": "boolean"
                        }
                    ],
                    "links": {
                        "notification_channels": [
                            "integer"
                        ],
                        "applications": [
                            "integer"
                        ],
                        "key_transactions": [
                            "integer"
                        ],
                        "servers": [
                            "integer"
                        ]
                    }
                }
            }

        """
        return self._get(
            url='{0}alert_policies/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )

    def update(self, id, policy_update):
        """
        This API endpoint allows you to update your alert policies.

        The input is expected to be in **JSON** format in the body
        parameters of the PUT request. The exact schema is defined below. Any
        extra parameters passed in the body **will be ignored** .::

            {
                "alert_policy": {
                    "name": str,
                    "enabled": bool,
                    "conditions": [
                        {
                            "id": int,
                            "threshold": float,
                            "trigger_minutes": int,
                            "enabled": bool
                        }
                    ],
                    "links": {
                        "notification_channels": [
                            int
                        ],
                        "applications": [
                            int
                        ],
                        "key_transactions": [
                            "int
                        ],
                        "servers": [
                            int
                        ]
                    }
                }
            }

        **NOTE:** When updating alertable and notification channel links, the
        list sent replaces the existing list. Invalid values will be ignored
        but an empty array will result in alertables/channels being reset.

        :type id: int
        :param id: Alert policy ID

        :type policy_update: dict
        :param policy_update: The json of the policy to update

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "alert_policy": {
                    "id": "integer",
                    "type": "string",
                    "name": "string",
                    "enabled": "boolean",
                    "conditions": [
                        {
                            "id": "integer",
                            "type": "string",
                            "severity": "string",
                            "threshold": "float",
                            "trigger_minutes": "integer",
                            "enabled": "boolean"
                        }
                    ],
                    "links": {
                        "notification_channels": [
                            "integer"
                        ],
                        "applications": [
                            "integer"
                        ],
                        "key_transactions": [
                            "integer"
                        ],
                        "servers": [
                            "integer"
                        ]
                    }
                }
            }

        """
        return self._put(
            url='{0}alert_policies/{1}.json'.format(self.URL, id),
            headers=self.headers,
            data=policy_update
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.components import Components


class NRComponentsTests(TestCase):
    def setUp(self):
        super(NRComponentsTests, self).setUp()
        self.components = Components(api_key='dummy_key')

        self.list_response = {
            "components": [
                {
                    "id": 2223333,
                    "name": "SendGrid",
                    "summary_metrics": []
                }
            ]
        }

        self.show_response = {
            'component': self.list_response['components'][0]
        }

        self.metric_name_response = {
            "metrics": [
                {
                    "name": "Component/Numeric/Compliance/Blocked[Emails/Day]",
                    "values": [
                        "average_value",
                        "min_value",
                        "max_value",
                        "sample_count",
                        "total_value"
                    ]
                },
            ]
        }

        self.metric_data_response = {
            "metric_data": {
                "from": "2014-06-28T16:15:30+00:00",
                "to": "2014-06-28T16:45:30+00:00",
                "metrics": [
                    {
                        "name": "Component/Numeric/Compliance/Blocked[Emails/Day]",
                        "timeslices": [
                            {
                                "from": "2014-06-28T16:14:00+00:00",
                                "to": "2014-06-28T16:43:59+00:00",
                                "values": {
                                    "average_value": 0,
                                    "min_value": 0,
                                    "max_value": 0,
                                    "sample_count": 5,
                                    "total_value": 0
                                }
                            }
                        ]
                    }
                ]
            }
        }

    @patch.object(requests, 'get')
    def test_list(self, mock_get):
        """
        Test components .list()
        """
        self.components.list(filter_name='SendGrid', page=0)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/components.json',
            headers=self.components.headers,
            params='filter[name]=SendGrid'
        )

    @patch.object(requests, 'get')
    def test_list_with_filter_ids(self, mock_get):
        """
        Test components .list() with filter_ids
        """
        self.components.list(filter_name='SendGrid', filter_ids=[2223333], page=0)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/components.json',
            headers=self.components.headers,
            params='filter[name]=SendGrid&filter[ids]=2223333'
        )

    @patch.object(requests, 'get')
    def test_show(self, mock_get):
        """
        Test components .show()
        """
        self.components.show(id=2223333)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/components/2223333.json',
            headers=self.components.headers,
        )

    @patch.object(requests, 'get')
    def test_metric_names(self, mock_get):
        """
        Test components .metric_names()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_name_response
        mock_get.return_value = mock_response

        response = self.components.metric_names(id=2223333)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_without_values(self, mock_get):
        """
        Test components .metric_data() without values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.components.metric_data(
            id=2223333,
            names=['Component/Numeric/Compliance/Blocked[Emails/Day]'],
            summarize=True
        )

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_metric_data_with_values(self, mock_get):
        """
        Test components .metric_data() with values param
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.metric_data_response
        mock_get.return_value = mock_response

        response = self.components.metric_data(
            id=2223333,
            names=['Component/Numeric/Compliance/Blocked[Emails/Day]'],
            values=['average_value'],
            summarize=True
        )

        self.assertIsInstance(response, dict)



import os
from datetime import time
from unittest import TestCase

from mock import patch, call, Mock
import requests

from newrelic_api.base import Resource
from newrelic_api.exceptions import ConfigurationException, NewRelicAPIServerException


class ResourceTests(TestCase):

    def setUp(self):
        self.TEST_URL = 'https://www.google.com'

    @patch.object(os.environ, 'get', spec_set=True)
    def test_resource_with_arg(self, os_environ_mock):
        """
        Test the api_key gets set by an argument
        """

        resource = Resource(api_key='123')

        self.assertFalse(os_environ_mock.called)

        self.assertEqual(resource.api_key, '123')

    @patch.object(os.environ, 'get', spec_set=True)
    def test_second_resource_from_os(self, os_environ_mock):
        """
        Test the api_key gets set by the OS by NEWRELIC_API_KEY
        """
        os_environ_mock.side_effect = (None, '123')

        resource = Resource()

        os_environ_mock.assert_has_calls(
            [
                call('NEW_RELIC_API_KEY'),
                call('NEWRELIC_API_KEY'),
            ]
        )

        self.assertEqual(resource.api_key, '123')

    @patch.object(os.environ, 'get', spec_set=True)
    def test_resource_from_os(self, os_environ_mock):
        """
        Test the api_key gets set by the OS
        """
        os_environ_mock.return_value = '123'

        resource = Resource()

        os_environ_mock.assert_called_once_with('NEW_RELIC_API_KEY')

        self.assertEqual(resource.api_key, '123')

    @patch.object(os.environ, 'get', spec_set=True)
    def test_resource_raises_error(self, os_environ_mock):
        """
        Test the api_key is not set
        """
        os_environ_mock.return_value = None

        with self.assertRaises(ConfigurationException):
            Resource()

        # os_environ_mock.assert_called_once_with('NEWRELIC_API_KEY')

    @patch.object(requests, 'get')
    def test_get_ok_with_links(self, mock_get):
        """
        Test ._get() handles an ok response with links
        """
        mock_response = Mock(
            name='response',
            ok=True,
            status_code=200,
            links={
                "last": {
                    "url": "https://api.newrelic.com/v2/servers.json?page=2",
                    "rel": "last"
                },
                "next": {
                    "url": "https://api.newrelic.com/v2/servers.json?page=2",
                    "rel": "next"
                }
            }
        )
        mock_response.json.return_value = {
            "servers": [
                {
                    "id": "integer",
                    "account_id": "integer",
                    "name": "string",
                    "host": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "summary": {
                        "cpu": "float",
                        "cpu_stolen": "float",
                        "disk_io": "float",
                        "memory": "float",
                        "memory_used": "integer",
                        "memory_total": "integer",
                        "fullest_disk": "float",
                        "fullest_disk_free": "integer"
                    }
                }
            ]
        }

        mock_get.return_value = mock_response

        resource = Resource(api_key='123')

        response = resource._get(url=self.TEST_URL)

        self.assertIn(
            'pages',
            list(response.keys())
        )

    @patch.object(requests, 'get')
    def test_get_ok_without_links(self, mock_get):
        """
        Test ._get() handles an ok response with links
        """
        mock_response = Mock(
            name='response',
            ok=True,
            status_code=200,
            links=None
        )
        mock_response.json.return_value = {
            "servers": [
                {
                    "id": "integer",
                    "account_id": "integer",
                    "name": "string",
                    "host": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "summary": {
                        "cpu": "float",
                        "cpu_stolen": "float",
                        "disk_io": "float",
                        "memory": "float",
                        "memory_used": "integer",
                        "memory_total": "integer",
                        "fullest_disk": "float",
                        "fullest_disk_free": "integer"
                    }
                }
            ]
        }

        mock_get.return_value = mock_response

        resource = Resource(api_key='123')

        response = resource._get(url=self.TEST_URL)

        self.assertNotIn(
            'pages',
            list(response.keys())
        )

    @patch.object(requests, 'get')
    def test_get_not_ok(self, mock_get):
        """
        Test ._get() handles a not ok response
        """
        mock_response = Mock(name='response', ok=False, status_code=500, text='Server Error')
        mock_get.return_value = mock_response

        resource = Resource(api_key='123')

        with self.assertRaises(NewRelicAPIServerException):
            resource._get(url=self.TEST_URL)

        mock_get.assert_called_once_with(
            url=self.TEST_URL,
        )

    @patch.object(requests, 'put')
    def test_put_not_ok(self, mock_put):
        """
        Test ._put() handles a not ok response
        """
        mock_response = Mock(name='response', ok=False, status_code=500, text='Server Error')
        mock_put.return_value = mock_response

        resource = Resource(api_key='123')

        with self.assertRaises(NewRelicAPIServerException):
            resource._put(url=self.TEST_URL)

        mock_put.assert_called_once_with(
            url=self.TEST_URL,
        )

    @patch.object(requests, 'post')
    def test_post_not_ok(self, mock_post):
        """
        Test ._post() handles a not ok response
        """
        mock_response = Mock(name='response', ok=False, status_code=500, text='Server Error')
        mock_post.return_value = mock_response

        resource = Resource(api_key='123')

        with self.assertRaises(NewRelicAPIServerException):
            resource._post(url=self.TEST_URL)

        mock_post.assert_called_once_with(
            url=self.TEST_URL,
        )

    @patch.object(requests, 'delete')
    def test_delete_not_ok(self, mock_delete):
        """
        Test ._delete() handles a not ok response
        """
        mock_response = Mock(name='response', ok=False, status_code=500, text='Server Error')
        mock_delete.return_value = mock_response

        resource = Resource(api_key='123')

        with self.assertRaises(NewRelicAPIServerException):
            resource._delete(url=self.TEST_URL)

        mock_delete.assert_called_once_with(
            url=self.TEST_URL,
        )

    def test_build_param_string(self):
        """
        Tests .build_param_string() returns the correct string
        """
        resource = Resource(api_key='123')

        test_params = [
            False,
            'filter[name]=dev',
            None,
            'page=1',
            0,
            [],
            {},
            time(0)
        ]

        param_str = resource.build_param_string(test_params)

        self.assertEqual(param_str, 'filter[name]=dev&page=1')



# flake8: noqa
from .version import __version__

from .alert_policies import AlertPolicies
from .applications import Applications
from .application_hosts import ApplicationHosts
from .application_instances import ApplicationInstances
from .components import Components
from .key_transactions import KeyTransactions
from .notification_channels import NotificationChannels
from .plugins import Plugins
from .servers import Servers
from .users import Users



from .base import Resource


class BrowserApplications(Resource):
    """
    An interface for interacting with the NewRelic Browser Application API.
    """
    def list(self, filter_name=None, filter_ids=None, page=None):
        """
        This API endpoint returns a list of the Browser Applications associated
            with your New Relic account.

        Browser Applications can be filtered by their name, or by the
            application IDs.

        :type filter_name: str
        :param filter_name: Filter by application name

        :type filter_ids: list of ints
        :param filter_ids: Filter by application ids

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "browser_applications": [
                    {
                        "id": "integer",
                        "name": "string",
                        "browser_monitoring_key": "string",
                        "loader_script": "string"
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/browser_applications.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/browser_applications.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}browser_applications.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def create(self, name):
        """
        This API endpoint allows you to create a standalone Browser Application

        :type name: str
        :param name: The name of the application

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "browser_application": {
                    "id": "integer",
                    "name": "string",
                    "browser_monitoring_key": "string",
                    "loader_script": "string"
                }
            }

        """

        data = {
            "browser_application": {
                "name": name
            }
        }

        return self._post(
            url='{0}browser_applications.json'.format(self.URL),
            headers=self.headers,
            data=data
        )



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.browser_applications import BrowserApplications


class NRBrowserApplicationsTests(TestCase):
    def setUp(self):
        super(NRBrowserApplicationsTests, self).setUp()
        self.browser_application = BrowserApplications(api_key='dummy_key')

        browser_application = {
            "id": 1234567,
            "name": "Account Global",
            "browser_monitoring_key": "313ed76e08",
            "loader_script": (
                "<script type=\"text/javascript\">"
                "\n</script>\n"
            )
        }
        self.list_success_response = {
            'browser_applications': [
                browser_application,
            ],
        }
        self.create_success_response = {
            'browser_application': browser_application,
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test browser_applications .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.browser_application.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_filters(self, mock_get):
        """
        Test browser_applications .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.list_success_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.browser_application.list(filter_ids=[1234567], filter_name='Account Global', page=1)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test browser_applications .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.browser_application.list()

    @patch.object(requests, 'post')
    def test_create_success(self, mock_post):
        """
        Test browser_applications .create() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.create_success_response
        mock_post.return_value = mock_response

        # Call the method
        response = self.browser_application.create(name='Account Global')

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'post')
    def test_create_failure(self, mock_post):
        """
        Test browser_applications .create() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_post.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.browser_application.create(name='Account Global')



# import multiprocessing to avoid this bug (http://bugs.python.org/issue15881#msg170215)
import multiprocessing
assert multiprocessing
import re
from setuptools import setup, find_packages


def get_version():
    """
    Extracts the version number from the version.py file.
    """
    VERSION_FILE = 'newrelic_api/version.py'
    mo = re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', open(VERSION_FILE, 'rt').read(), re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError('Unable to find version string in {0}.'.format(VERSION_FILE))


setup(
    name='newrelic-api',
    version=get_version(),
    description='A python interface to the New Relic API v2',
    long_description=open('README.rst').read(),
    url='https://github.com/ambitioninc/newrelic-api',
    author='Micah Hausler',
    author_email='opensource@ambition.com',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    license='MIT',
    install_requires=[
        'requests>=2.0.0'
    ],
    test_suite='nose.collector',
    tests_require=[
        'coverage>=3.7.1',
        'mock>=1.0.1',
        'nose>=1.3.0',
    ],
    include_package_data=True,
    zip_safe=True,
)



from .base import Resource


class Applications(Resource):
    """
    An interface for interacting with the NewRelic application API.
    """
    def list(
            self, filter_name=None, filter_ids=None, filter_language=None,
            page=None):
        """
        This API endpoint returns a paginated list of the Applications
        associated with your New Relic account. Applications can be filtered
        by their name, the list of application IDs or the application language
        as reported by the agents.

        :type filter_name: str
        :param filter_name: Filter by application name

        :type filter_ids: list of ints
        :param filter_ids: Filter by application ids

        :type filter_language: str
        :param filter_language: Filter by application language

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "applications": [
                    {
                        "id": "integer",
                        "name": "string",
                        "language": "string",
                        "health_status": "string",
                        "reporting": "boolean",
                        "last_reported_at": "time",
                        "application_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "error_rate": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "end_user_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "settings": {
                            "app_apdex_threshold": "float",
                            "end_user_apdex_threshold": "float",
                            "enable_real_user_monitoring": "boolean",
                            "use_server_side_config": "boolean"
                        },
                        "links": {
                            "servers": [
                                "integer"
                            ],
                            "application_hosts": [
                                "integer"
                            ],
                            "application_instances": [
                                "integer"
                            ]
                        }
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/applications.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/applications.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[language]={0}'.format(','.join(filter_language)) if filter_language else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{0}applications.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single Application, identified its ID.

        :type id: int
        :param id: Application ID

        :rtype: dict
        :return: The JSON response of the API.

        ::

                {
                    "application": {
                        "id": "integer",
                        "name": "string",
                        "language": "string",
                        "health_status": "string",
                        "reporting": "boolean",
                        "last_reported_at": "time",
                        "application_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "error_rate": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "end_user_summary": {
                            "response_time": "float",
                            "throughput": "float",
                            "apdex_target": "float",
                            "apdex_score": "float"
                        },
                        "settings": {
                            "app_apdex_threshold": "float",
                            "end_user_apdex_threshold": "float",
                            "enable_real_user_monitoring": "boolean",
                            "use_server_side_config": "boolean"
                        },
                        "links": {
                            "servers": [
                                "integer"
                            ],
                            "application_hosts": [
                                "integer"
                            ],
                            "application_instances": [
                                "integer"
                            ]
                        }
                    }
                }

        """
        return self._get(
            url='{0}applications/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )

    def update(
            self, id, name=None, app_apdex_threshold=None, end_user_apdex_threshold=None,
            enable_real_user_monitoring=None):
        """
        Updates any of the optional parameters of the application

        :type id: int
        :param id: Application ID

        :type name: str
        :param name: The name of the application

        :type app_apdex_threshold: float
        :param app_apdex_threshold: Application apdex threshold to update

        :type end_user_apdex_threshold: float
        :param end_user_apdex_threshold: End user apdex threshold to update

        :type enable_real_user_monitoring: bool
        :param enable_real_user_monitoring: Whether to enable real user
            monitoring

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "application": {
                    "id": "integer",
                    "name": "string",
                    "language": "string",
                    "health_status": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "application_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "error_rate": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "end_user_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "settings": {
                        "app_apdex_threshold": "float",
                        "end_user_apdex_threshold": "float",
                        "enable_real_user_monitoring": "boolean",
                        "use_server_side_config": "boolean"
                    },
                    "links": {
                        "servers": [
                            "integer"
                        ],
                        "application_hosts": [
                            "integer"
                        ],
                        "application_instances": [
                            "integer"
                        ]
                    }
                }
            }

        """
        nr_data = self.show(id)['application']

        data = {
            'application': {
                'name': name or nr_data['name'],
                'settings': {
                    'app_apdex_threshold':
                        app_apdex_threshold or nr_data['settings']['app_apdex_threshold'],
                    'end_user_apdex_threshold':
                        end_user_apdex_threshold or nr_data['settings']['end_user_apdex_threshold'],
                    'enable_real_user_monitoring':
                        enable_real_user_monitoring or nr_data['settings']['enable_real_user_monitoring']
                }
            }
        }

        return self._put(
            url='{0}{1}/{2}.json'.format(
                self.URL,
                'applications',
                id),
            headers=self.headers,
            data=data
        )

    def delete(self, id):
        """
        This API endpoint deletes an application and all of its reported data.

        WARNING: Only applications that have stopped reporting can be deleted.
            This is an irreversible process which will delete all reported
            data for this application.

        :type id: int
        :param id: Application ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "application": {
                    "id": "integer",
                    "name": "string",
                    "language": "string",
                    "health_status": "string",
                    "reporting": "boolean",
                    "last_reported_at": "time",
                    "application_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "error_rate": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "end_user_summary": {
                        "response_time": "float",
                        "throughput": "float",
                        "apdex_target": "float",
                        "apdex_score": "float"
                    },
                    "settings": {
                        "app_apdex_threshold": "float",
                        "end_user_apdex_threshold": "float",
                        "enable_real_user_monitoring": "boolean",
                        "use_server_side_config": "boolean"
                    },
                    "links": {
                        "servers": [
                            "integer"
                        ],
                        "application_hosts": [
                            "integer"
                        ],
                        "application_instances": [
                            "integer"
                        ]
                    }
                }
            }

        """
        return self._delete(
            url='{0}applications/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )

    def metric_names(self, id, name=None, page=None):
        """
        Return a list of known metrics and their value names for the given resource.

        :type id: int
        :param id: Application ID

        :type name: str
        :param name: Filter metrics by name

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "metrics": [
                    {
                        "name": "string",
                        "values": [
                            "string"
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/applications/{application_id}/metrics.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/applications/{application_id}/metrics.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        params = [
            'name={0}'.format(name) if name else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}applications/{1}/metrics.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )

    def metric_data(
            self, id, names, values=None, from_dt=None, to_dt=None,
            summarize=False):
        """
        This API endpoint returns a list of values for each of the requested
        metrics. The list of available metrics can be returned using the Metric
        Name API endpoint. Metric data can be filtered by a number of
        parameters, including multiple names and values, and by time range.
        Metric names and values will be matched intelligently in the
        background. You can also retrieve a summarized data point across the
        entire time range selected by using the summarize parameter.

        **Note** All times sent and received are formatted in UTC. The default
        time range is the last 30 minutes.

        :type id: int
        :param id: Application ID

        :type names: list of str
        :param names: Retrieve specific metrics by name

        :type values: list of str
        :param values: Retrieve specific metric values

        :type from_dt: datetime
        :param from_dt: Retrieve metrics after this time

        :type to_dt: datetime
        :param to_dt: Retrieve metrics before this time

        :type summarize: bool
        :param summarize: Summarize the data

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "metric_data": {
                    "from": "time",
                    "to": "time",
                    "metrics": [
                        {
                            "name": "string",
                            "timeslices": [
                                {
                                    "from": "time",
                                    "to": "time",
                                    "values": "hash"
                                }
                            ]
                        }
                    ]
                }
            }

        """
        params = [
            'from={0}'.format(from_dt) if from_dt else None,
            'to={0}'.format(to_dt) if to_dt else None,
            'summarize=true' if summarize else None
        ]

        params += ['names[]={0}'.format(name) for name in names]
        if values:
            params += ['values[]={0}'.format(value) for value in values]

        return self._get(
            url='{0}applications/{1}/metrics/data.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )



from .base import Resource


class Components(Resource):
    """
    An interface for interacting with the NewRelic component API.
    """
    def list(
            self, filter_name=None, filter_ids=None, filter_plugin_id=None,
            page=None):
        """
        This API endpoint returns a paginated list of the Components
        associated with your New Relic account. Components can be filtered
        by their name or by a list of component IDs.

        :type filter_name: str
        :param filter_name: Filter by component name

        :type filter_ids: list of ints
        :param filter_ids: Filter by component ids

        :type filter_plugin_id: int
        :param filter_plugin_id: Filter components by the plugin

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results

        ::

            {
                "components": [
                    {
                        "id": "integer",
                        "name": "string",
                        "summary_metrics": [
                            {
                                "id": "integer",
                                "name": "string",
                                "metric": "string",
                                "value_function": "string",
                                "thresholds": {
                                    "caution": "float",
                                    "critical": "float"
                                },
                                "values": {
                                    "raw": "float",
                                    "formatted": "string"
                                }
                            }
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/components.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/components.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        filters = [
            'filter[name]={0}'.format(filter_name) if filter_name else None,
            'filter[ids]={0}'.format(
                ','.join([str(app_id) for app_id in filter_ids])
            ) if filter_ids else None,
            'filter[plugin_id]={0}'.format(filter_plugin_id) if filter_plugin_id else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{0}components.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single component, identified its ID.

        :type id: int
        :param id: Component ID

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "component": {
                    "id": "integer",
                    "name": "string",
                    "summary_metrics": [
                        {
                            "id": "integer",
                            "name": "string",
                            "metric": "string",
                            "value_function": "string",
                            "thresholds": {
                                "caution": "float",
                                "critical": "float"
                            },
                            "values": {
                                "raw": "float",
                                "formatted": "string"
                            }
                        }
                    ]
                }
            }

        """
        return self._get(
            url='{0}components/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )

    def metric_names(self, id, name=None, page=None):
        """
        Return a list of known metrics and their value names for the given resource.

        :type id: int
        :param id: Component ID

        :type name: str
        :param name: Filter metrics by name

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "metrics": [
                    {
                        "name": "string",
                        "values": [
                            "string"
                        ]
                    }
                ],
                "pages": {
                    "last": {
                        "url": "https://api.newrelic.com/v2/components/{component_id}/metrics.json?page=2",
                        "rel": "last"
                    },
                    "next": {
                        "url": "https://api.newrelic.com/v2/components/{component_id}/metrics.json?page=2",
                        "rel": "next"
                    }
                }
            }

        """
        params = [
            'name={0}'.format(name) if name else None,
            'page={0}'.format(page) if page else None
        ]

        return self._get(
            url='{0}components/{1}/metrics.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )

    def metric_data(
            self, id, names, values=None, from_dt=None, to_dt=None,
            summarize=False):
        """
        This API endpoint returns a list of values for each of the requested
        metrics. The list of available metrics can be returned using the Metric
        Name API endpoint.

        Metric data can be filtered by a number of parameters, including
        multiple names and values, and by time range. Metric names and values
        will be matched intelligently in the background.

        You can also retrieve a summarized data point across the entire time
        range selected by using the summarize parameter.

        **Note** All times sent and received are formatted in UTC. The default
        time range is the last 30 minutes.

        :type id: int
        :param id: Component ID

        :type names: list of str
        :param names: Retrieve specific metrics by name

        :type values: list of str
        :param values: Retrieve specific metric values

        :type from_dt: datetime
        :param from_dt: Retrieve metrics after this time

        :type to_dt: datetime
        :param to_dt: Retrieve metrics before this time

        :type summarize: bool
        :param summarize: Summarize the data

        :rtype: dict
        :return: The JSON response of the API

        ::

            {
                "metric_data": {
                    "from": "time",
                    "to": "time",
                    "metrics": [
                        {
                            "name": "string",
                            "timeslices": [
                                {
                                    "from": "time",
                                    "to": "time",
                                    "values": "hash"
                                }
                            ]
                        }
                    ]
                }
            }

        """
        params = [
            'from={0}'.format(from_dt) if from_dt else None,
            'to={0}'.format(to_dt) if to_dt else None,
            'summarize=true' if summarize else None
        ]

        params += ['names[]={0}'.format(name) for name in names]
        if values:
            params += ['values[]={0}'.format(value) for value in values]

        return self._get(
            url='{0}components/{1}/metrics/data.json'.format(self.URL, id),
            headers=self.headers,
            params=self.build_param_string(params)
        )



import os
import json

import requests

from newrelic_api.exceptions import ConfigurationException, NewRelicAPIServerException


class Resource(object):
    """
    A base class for API resources
    """
    URL = 'https://api.newrelic.com/v2/'

    def __init__(self, api_key=None):
        """
        :type api_key: str
        :param api_key: The API key. If no key is passed, the environment
            variable NEW_RELIC_API_KEY is used.
        :raises: If the api_key parameter is not present, and no environment
            variable is present, a :class:`newrelic_api.exceptions.ConfigurationException`
            is raised.
        """
        self.api_key = api_key or os.environ.get('NEW_RELIC_API_KEY') or os.environ.get('NEWRELIC_API_KEY')

        if not self.api_key:
            raise ConfigurationException('NEW_RELIC_API_KEY or NEWRELIC_API_KEY not present in environment!')

        self.headers = {
            'Content-type': 'application/json',
            'X-Api-Key': self.api_key,
        }

    def _get(self, *args, **kwargs):
        """
        A wrapper for getting things

        :returns: The response of your get
        :rtype: dict

        :raises: This will raise a
            :class:`NewRelicAPIServerException<newrelic_api.exceptions.NewRelicAPIServerException>`
            if there is an error from New Relic
        """
        response = requests.get(*args, **kwargs)
        if not response.ok:
            raise NewRelicAPIServerException('{}: {}'.format(response.status_code, response.text))

        json_response = response.json()

        if response.links:
            json_response['pages'] = response.links

        return json_response

    def _put(self, *args, **kwargs):
        """
        A wrapper for putting things. It will also json encode your 'data' parameter

        :returns: The response of your put
        :rtype: dict

        :raises: This will raise a
            :class:`NewRelicAPIServerException<newrelic_api.exceptions.NewRelicAPIServerException>`
            if there is an error from New Relic
        """
        if 'data' in kwargs:
            kwargs['data'] = json.dumps(kwargs['data'])
        response = requests.put(*args, **kwargs)
        if not response.ok:
            raise NewRelicAPIServerException('{}: {}'.format(response.status_code, response.text))
        return response.json()

    def _post(self, *args, **kwargs):
        """
        A wrapper for posting things. It will also json encode your 'data' parameter

        :returns: The response of your post
        :rtype: dict

        :raises: This will raise a
            :class:`NewRelicAPIServerException<newrelic_api.exceptions.NewRelicAPIServerException>`
            if there is an error from New Relic
        """
        if 'data' in kwargs:
            kwargs['data'] = json.dumps(kwargs['data'])
        response = requests.post(*args, **kwargs)
        if not response.ok:
            raise NewRelicAPIServerException('{}: {}'.format(response.status_code, response.text))
        return response.json()

    def _delete(self, *args, **kwargs):
        """
        A wrapper for deleting things

        :returns: The response of your delete
        :rtype: dict

        :raises: This will raise a
            :class:`NewRelicAPIServerException<newrelic_api.exceptions.NewRelicAPIServerException>`
            if there is an error from New Relic
        """
        response = requests.delete(*args, **kwargs)
        if not response.ok:
            raise NewRelicAPIServerException('{}: {}'.format(response.status_code, response.text))
        return response.json()

    def build_param_string(self, params):
        """
        This is a simple helper method to build a parameter string. It joins
        all list elements that evaluate to True with an ampersand, '&'

        .. code-block:: python

            >>> parameters = Resource().build_param_string(['filter[name]=dev', None, 'page=1'])
            >>> print parameters
            filter[name]=dev&page=1

        :type params: list
        :param params: The parameters to build a string with

        :rtype: str
        :return: The compiled parameter string
        """
        return '&'.join([p for p in params if p])



from unittest import TestCase

from mock import patch, Mock
import requests

from newrelic_api.key_transactions import KeyTransactions


class NRKeyTransactionsTests(TestCase):
    def setUp(self):
        super(NRKeyTransactionsTests, self).setUp()
        self.key_transactions = KeyTransactions(api_key='dummy_key')

        self.key_transactions_list_response = {
            "key_transactions": [
                {
                    "id": 333112,
                    "name": "login",
                    "transaction_name": "Login Page",
                    "application_summary": {
                        "response_time": 170,
                        "throughput": 3,
                        "error_rate": 0,
                        "apdex_target": 0,
                        "apdex_score": 1
                    },
                    "end_user_summary": {
                        "response_time": 300.0,
                        "throughput": 4,
                        "apdex_target": 0,
                        "apdex_score": 1
                    },
                    "links": {
                        "application": 123456
                    }
                }
            ]
        }
        self.key_transactions_show_response = {
            'key_transaction': self.key_transactions_list_response['key_transactions'][0]
        }

    @patch.object(requests, 'get')
    def test_list_success(self, mock_get):
        """
        Test key transactions .list()
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.key_transactions_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.key_transactions.list()

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_list_success_with_filter_ids(self, mock_get):
        """
        Test key transactions .list() with filter_ids
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.key_transactions_list_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.key_transactions.list(filter_ids=[333112])

        self.assertIsInstance(response, dict)

        mock_get.assert_called_once_with(
            url='https://api.newrelic.com/v2/key_transactions.json',
            headers=self.key_transactions.headers,
            params='filter[ids]=333112'
        )

    @patch.object(requests, 'get')
    def test_list_failure(self, mock_get):
        """
        Test key transactions .list() failure case
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.key_transactions.list()

    @patch.object(requests, 'get')
    def test_show_success(self, mock_get):
        """
        Test key transactions .show() success
        """
        mock_response = Mock(name='response')
        mock_response.json.return_value = self.key_transactions_show_response
        mock_get.return_value = mock_response

        # Call the method
        response = self.key_transactions.show(id=333112)

        self.assertIsInstance(response, dict)

    @patch.object(requests, 'get')
    def test_show_failure(self, mock_get):
        """
        Test key transactions .show() failure
        """
        mock_response = Mock(name='response')
        mock_response.json.side_effect = ValueError('No JSON object could be decoded')
        mock_get.return_value = mock_response

        with self.assertRaises(ValueError):
            # Call the method
            self.key_transactions.show(id=333114)



from .base import Resource


class NotificationChannels(Resource):
    """
    An interface for interacting with the NewRelic Notification Channels API.
    """
    def list(self, filter_type=None, filter_ids=None, page=None):
        """
        This API endpoint returns a paginated list of the notification channels
        associated with your New Relic account.

        Notification channels can be filtered by their type or a list of IDs.

        :type filter_type: list of str
        :param filter_type: Filter by notification channel types

        :type filter_ids: list of int
        :param filter_ids: Filter by notification channel ids

        :type page: int
        :param page: Pagination index

        :rtype: dict
        :return: The JSON response of the API, with an additional 'pages' key
            if there are paginated results
        """
        filters = [
            'filter[type]={0}'.format(','.join(filter_type)) if filter_type else None,
            'filter[ids]={0}'.format(','.join([str(app_id) for app_id in filter_ids])) if filter_ids else None,
            'page={0}'.format(page) if page else None
        ]
        return self._get(
            url='{0}notification_channels.json'.format(self.URL),
            headers=self.headers,
            params=self.build_param_string(filters)
        )

    def show(self, id):
        """
        This API endpoint returns a single notification channel, identified by
        ID.

        :type id: int
        :param id: notification channel ID

        :rtype: dict
        :return: The JSON response of the API
        """
        return self._get(
            url='{0}notification_channels/{1}.json'.format(self.URL, id),
            headers=self.headers,
        )
